package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.PromoMediaAddUpdateParam;
import com.zanxiang.manage.domain.params.PromoMediaListParam;
import com.zanxiang.manage.domain.vo.PromoMediaListVO;
import com.zanxiang.manage.service.IPromoMediaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author : lingfeng
 * @time : 2022-11-18
 * @description : 推广媒体
 */
@Api(tags = "推广媒体接口")
@RestController
@RequestMapping("/promo/media")
public class PromoMediaController {

    @Autowired
    private IPromoMediaService promoMediaService;

    @ApiOperation(value = "查询推广媒体列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "manage:promoMedia:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PromoMediaListVO.class)})
    public ResultVO<IPage<PromoMediaListVO>> list(@Validated @RequestBody PromoMediaListParam param) {
        return ResultVO.ok(promoMediaService.list(param));
    }

    @ApiOperation(value = "推广媒体添加或修改")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "manage:promoMedia:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addOrUpdate(@Validated @RequestBody PromoMediaAddUpdateParam param) {
        return ResultVO.ok(promoMediaService.addOrUpdate(param));
    }

    @ApiOperation(value = "推广媒体状态变更")
    @PatchMapping(value = "/status/update")
    @PreAuthorize(permissionKey = "manage:promoMedia:statusUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> statusUpdate(@RequestParam Long id, @RequestParam Integer status) {
        return new ResultVO<>(promoMediaService.statusUpdate(id, status));
    }
}
