package com.zanxiang.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.oceanengine.track.base.pojo.PlatformServer;
import com.zanxiang.advertising.oceanengine.track.base.pojo.dto.AppOrderRpcDTO;
import com.zanxiang.advertising.oceanengine.track.base.pojo.rpc.IAppOrderRpc;
import com.zanxiang.advertising.oceanengine.track.base.pojo.vo.AppReportLogRpcVO;
import com.zanxiang.common.enums.CallBackEnum;
import com.zanxiang.common.enums.PayWayEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.*;
import com.zanxiang.manage.domain.params.OrderParam;
import com.zanxiang.manage.domain.params.UserOrderListParam;
import com.zanxiang.manage.domain.vo.OrderListVO;
import com.zanxiang.manage.domain.vo.OrderVO;
import com.zanxiang.manage.domain.vo.UserOrderListVO;
import com.zanxiang.manage.service.*;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.mapper.OrderMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.DubboReference;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-07-05
 * @description : 用户订单
 */
@Slf4j
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IGameService gameService;

    @Autowired
    private IGameCategoryService gameCategoryService;

    @Autowired
    private IPromoChannelService promoChannelService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IPromoAccountService promoAccountService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IPayWayService payWayService;

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IPayDeviceService payDeviceService;

    @DubboReference(providedBy = PlatformServer.SERVER_DUBBO_NAME)
    private IAppOrderRpc appOrderRpc;

    @Override
    public Boolean callBack(String orderId) {
        Order order = this.getOne(new LambdaQueryWrapper<Order>().eq(Order::getOrderId, orderId));
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        if (Objects.equals(order.getCallBackStatus(), CallBackEnum.SUCCESS_CALL_BACK.getCallBackStatus())) {
            throw new BaseException("订单已经上报, 请勿重复操作");
        }
        AppOrderRpcDTO dto = AppOrderRpcDTO.builder()
                .appKey(String.valueOf(order.getGameId()))
                .orderId(order.getOrderId())
                .userId(String.valueOf(order.getUserId()))
                .amount(order.getAmount().multiply(new BigDecimal("100")).longValue())
                .realAmount(order.getRealAmount().multiply(new BigDecimal("100")).longValue())
                .tranNo(order.getMerchantOrderNo())
                .orderStatus(order.getStatus())
                .payType(PayWayEnum.getByPayType(order.getPayWayId()))
                .payTime(order.getPayTime())
                .createTime(order.getCreateTime())
                .build();
        //调接口
        ResultVO<AppReportLogRpcVO> result;
        try {
            result = appOrderRpc.order(dto);
        } catch (Exception e) {
            log.error("订单回传上报异常, order : {}, e : {}", JsonUtil.toString(order), e.getMessage());
            throw new BaseException("订单回传上报异常");
        }
        log.error("订单回传结果, orderId : {}, result : {}", order.getOrderId(), JsonUtil.toString(result));
        String adId = "AD_ID";
        Integer callBackStatus = CallBackEnum.FAIL_CALL_BACK.getCallBackStatus();
        //成功
        if (result.isSuccess() && result.getData() != null && result.getData().getAid() != null) {
            adId = result.getData().getAid().toString();
            callBackStatus = CallBackEnum.SUCCESS_CALL_BACK.getCallBackStatus();
        }
        //更新数据库
        return orderService.update(new LambdaUpdateWrapper<Order>()
                .set(Order::getAdId, adId)
                .set(Order::getCallBackStatus, callBackStatus)
                .eq(Order::getOrderId, order.getOrderId()));
    }

    @Override
    public IPage<UserOrderListVO> orderList(UserOrderListParam param) {
        //执行查询
        return page(param.toPage(), new QueryWrapper<Order>().lambda()
                .eq(Order::getUserId, param.getUserId())
                .orderByDesc(Order::getCreateTime)
        ).convert(this::toVo);
    }

    private UserOrderListVO toVo(Order order) {
        if (order == null) {
            return null;
        }
        UserOrderListVO orderListVO = BeanUtils.copy(order, UserOrderListVO.class);
        orderListVO.setId(order.getOrderId());
        //查询角色名称
        GameUserRoleDTO gameUserRoleDTO = gameUserRoleService.getByRoleId(order.getRoleId());
        if (gameUserRoleDTO != null) {
            orderListVO.setRoleName(gameUserRoleDTO.getRoleName());
        }
        //设置游戏名称
        GameDTO gameDTO = gameService.getById(order.getGameId());
        if (gameDTO == null) {
            return orderListVO;
        }
        //设置游戏列表
        GameCategoryDTO gameCategoryDTO = gameCategoryService.getById(gameDTO.getCategory());
        if (gameCategoryDTO != null) {
            orderListVO.setGameCategoryName(gameCategoryDTO.getName());
        }
        orderListVO.setGameName(gameDTO.getName());
        return orderListVO;
    }

    @Override
    public OrderListVO orderList(OrderParam param) {
        //渠道列表
        List<Long> agentIds;
        if (param.getChannelId() == null) {
            agentIds = promoChannelService.listByAccountOrPitcherId(param.getAccountId(), param.getPitcherId());
        } else {
            agentIds = Collections.singletonList(param.getChannelId());
        }
        //根据条件, 匹配渠道
        if (agentIds != null && agentIds.isEmpty()) {
            return new OrderListVO(param.toPage().getSize());
        }
        IPage<OrderVO> page = page(param.toPage(), getListWrapper(param, agentIds, "*")).convert(this::orderToVo);
        if (page.getTotal() == 0) {
            return new OrderListVO(param.toPage().getSize());
        }
        //统计：订单金额与实付金额
        Order total = super.getOne(getListWrapper(param, agentIds, "IFNULL(SUM(amount),0) amount, IFNULL(SUM(real_amount),0) realAmount"));
        HashMap<String, BigDecimal> totalData = new HashMap<>();
        if (total == null) {
            totalData.put("totalOrderAmount", new BigDecimal("0.00"));
            totalData.put("totalRealAmount", new BigDecimal("0.00"));
        } else {
            totalData.put("totalOrderAmount", total.getAmount());
            totalData.put("totalRealAmount", total.getRealAmount());
        }
        //分页数据
        List<OrderVO> orderListVOList = page.getRecords();
        //cp信息
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        //玩家信息
        List<Long> userIdList = orderListVOList.stream().map(OrderVO::getUserId).collect(Collectors.toList());
        Map<Long, UserDTO> userMap = userService.getByUserIds(userIdList);
        //渠道信息
        List<Long> agentIdList = orderListVOList.stream().map(OrderVO::getAgentId).collect(Collectors.toList());
        Map<Long, PromoChannelDTO> channelMap = promoChannelService.promoChannelMap(agentIdList);
        //推广账号信息
        Map<Long, PromoAccountDTO> promoAccountMap = promoAccountService.promoAccountMap();
        //游戏信息
        List<Long> gameIds = orderListVOList.stream().map(OrderVO::getGameId).collect(Collectors.toList());
        List<Long> regGameIds = orderListVOList.stream().map(OrderVO::getRegGameId).collect(Collectors.toList());
        Set<Long> gameIdSet = new HashSet<>();
        gameIdSet.addAll(gameIds);
        gameIdSet.addAll(regGameIds);
        Map<Long, GameDTO> gameMap = gameService.gameMap(gameIdSet);
        //游戏分类信息
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        //支付渠道列表
        Map<Long, PayWayDTO> payWayMap = payWayService.payWayMap();
        //支付方式列表
        Map<Long, PayDeviceDTO> payDeviceMap = payDeviceService.payDeviceMap();
        //字段信息补充更新
        orderListVOList.forEach(vo -> this.update(vo, userMap, channelMap, gameMap, gameCategoryMap, cpMap,
                promoAccountMap, payWayMap, payDeviceMap));
        //设置返回
        OrderListVO orderListVO = new OrderListVO();
        orderListVO.setSize(page.getSize());
        orderListVO.setTotal(page.getTotal());
        orderListVO.setCurrent(page.getCurrent());
        orderListVO.setPages(page.getPages());
        orderListVO.setData(page.getRecords());
        orderListVO.setTotalData(totalData);
        return orderListVO;
    }

    private void update(OrderVO orderVO, Map<Long, UserDTO> userMap, Map<Long, PromoChannelDTO> channelMap,
                        Map<Long, GameDTO> gameMap, Map<Long, GameCategoryDTO> gameCategoryMap, Map<Long, CpDTO> cpMap,
                        Map<Long, PromoAccountDTO> promoAccountMap, Map<Long, PayWayDTO> payWayMap, Map<Long, PayDeviceDTO> payDeviceMap) {
        orderVO.setDateTime(orderVO.getCreateTime().toLocalDate());
        //用户信息补充
        UserDTO userDTO = userMap.get(orderVO.getUserId());
        if (userDTO != null) {
            //注册渠道
            PromoChannelDTO promoChannelDTO = channelMap.get(userDTO.getAgentId());
            if (promoChannelDTO != null) {
                orderVO.setRegAgentId(userDTO.getAgentId());
                orderVO.setRegChannel(promoChannelDTO.getChannel());
            }
        }
        //渠道信息补充
        PromoChannelDTO promoChannelDTO = channelMap.get(orderVO.getAgentId());
        if (promoChannelDTO != null) {
            orderVO.setChannel(promoChannelDTO.getChannel());
            PromoAccountDTO promoAccountDTO = promoAccountMap.get(promoChannelDTO.getAccountId());
            if (promoAccountDTO != null) {
                orderVO.setAccountName(promoAccountDTO.getName());
                orderVO.setAccountId(promoAccountDTO.getId());
                orderVO.setPitcherId(promoAccountDTO.getPitcherId());
                orderVO.setPitcherName(promoAccountDTO.getPitcherName());
                orderVO.setMediaId(promoAccountDTO.getMediaId());
                orderVO.setMediaName(promoAccountDTO.getMediaName());
            }
        }
        //注册游戏
        GameDTO regGameDTO = gameMap.get(orderVO.getRegGameId());
        if (regGameDTO != null) {
            orderVO.setRegGameName(regGameDTO.getName());
            //注册游戏应用类型
            GameCategoryDTO gameCategoryDTO = gameCategoryMap.get(regGameDTO.getCategory());
            if (gameCategoryDTO != null) {
                orderVO.setRegGameCategoryId(gameCategoryDTO.getId());
                orderVO.setRegGameCategoryName(gameCategoryDTO.getName());
            }
        }
        //CP
        CpDTO cpDTO = cpMap.get(orderVO.getCpId());
        if (cpDTO != null) {
            orderVO.setCpName(cpDTO.getCpName());
        }
        //充值游戏
        GameDTO gameDTO = gameMap.get(orderVO.getGameId());
        if (gameDTO != null) {
            orderVO.setGameName(gameDTO.getName());
            //注册游戏应用类型
            GameCategoryDTO gameCategoryDTO = gameCategoryMap.get(gameDTO.getCategory());
            if (gameCategoryDTO != null) {
                orderVO.setGameCategory(gameCategoryDTO.getName());
                orderVO.setGameCategoryId(gameCategoryDTO.getId());
            }
        }
        //支付渠道
        if (payWayMap.get(orderVO.getPayWayId()) != null) {
            orderVO.setPayWay(payWayMap.get(orderVO.getPayWayId()).getPayWayName());
        }
        //支付场景
        if (payDeviceMap.get(orderVO.getPayDeviceId()) != null) {
            orderVO.setPayDeviceName(payDeviceMap.get(orderVO.getPayDeviceId()).getPayDeviceName());
        }
    }

    private LambdaQueryWrapper<Order> getListWrapper(OrderParam param, List<Long> agentIdList, String select) {
        return new QueryWrapper<Order>().select(select).lambda()
                .eq(Strings.isNotBlank(param.getOrderId()), Order::getOrderId, param.getOrderId())
                .eq(Strings.isNotBlank(param.getCpOrderId()), Order::getCpOrderId, param.getCpOrderId())
                .eq(Strings.isNotBlank(param.getMerchantOrderNo()), Order::getMerchantOrderNo, param.getMerchantOrderNo())
                .eq(Strings.isNotBlank(param.getUserId()), Order::getUserId, param.getUserId())
                .eq(Strings.isNotBlank(param.getUsername()), Order::getUsername, param.getUsername())
                .eq(Strings.isNotBlank(param.getRoleName()), Order::getRoleName, param.getRoleName())
                .ge(param.getRegTimeStart() != null, Order::getRegTime, param.getRegTimeStart() == null ? null : LocalDateTime.of(param.getRegTimeStart(), LocalTime.MIN))
                .le(param.getRegTimeEnd() != null, Order::getRegTime, param.getRegTimeEnd() == null ? null : LocalDateTime.of(param.getRegTimeEnd(), LocalTime.MAX))
                .eq(param.getCpId() != null, Order::getCpId, param.getCpId())
                .eq(param.getRegGameId() != null, Order::getRegGameId, param.getRegGameId())
                .eq(param.getGameId() != null, Order::getGameId, param.getGameId())
                .in(CollectionUtils.isNotEmpty(agentIdList), Order::getAgentId, agentIdList)
                .eq(Strings.isNotBlank(param.getDeviceSystem()), Order::getDeviceSystem, param.getDeviceSystem())
                .eq(param.getIsSwitch() != null, Order::getIsSwitch, param.getIsSwitch())
                .eq(Strings.isNotBlank(param.getProductName()), Order::getProductName, param.getProductName())
                .eq(Strings.isNotBlank(param.getPayWayId()), Order::getPayWayId, param.getPayWayId())
                .eq(Strings.isNotBlank(param.getMerchantNo()), Order::getMerchantNo, param.getMerchantNo())
                .eq(param.getStatus() != null, Order::getStatus, param.getStatus())
                .eq(param.getCpStatus() != null, Order::getCpStatus, param.getCpStatus())
                .eq(param.getIsFirstRecharge() != null, Order::getIsFirstRecharge, param.getIsFirstRecharge())
                .eq(param.getCallBackStatus() != null, Order::getCallBackStatus, param.getCallBackStatus())
                .ge(param.getBeginDate() != null, Order::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, Order::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .orderByDesc(Order::getCreateTime);
    }


    private OrderVO orderToVo(Order order) {
        if (Objects.isNull(order)) {
            return null;
        }
        return BeanUtils.copy(order, OrderVO.class);
    }
}
